from bd2k.util.xml.builder import E
from cgcloud.core.agent_box import AgentBox
from cgcloud.lib.util import snake_to_camel, UserError
from cgcloud.fabric.operations import sudo
from cgcloud.core.box import fabric_task
from cgcloud.core.source_control_client import SourceControlClient
from cgcloud.jenkins.jenkins_master import Jenkins, JenkinsMaster

build_dir = '/home/jenkins/builds'


class JenkinsSlave( SourceControlClient, AgentBox ):
    """
    A box that represents EC2 instances which can serve as a Jenkins build agent. This class is
    typically used as a mix-in.
    """

    def other_accounts( self ):
        return super( JenkinsSlave, self ).other_accounts( ) + [ Jenkins.user ]

    def default_account( self ):
        return Jenkins.user

    def _post_install_packages( self ):
        super( JenkinsSlave, self )._post_install_packages( )
        self._setup_build_user( )

    # TODO: We should probably remove this and let the agent take care of it

    def __get_master_pubkey( self ):
        ec2_keypair_name = JenkinsMaster.ec2_keypair_name( self.ctx )
        ec2_keypair = self.ctx.ec2.get_key_pair( ec2_keypair_name )
        if ec2_keypair is None:
            raise UserError( "Missing EC2 keypair named '%s'. You must create the master before "
                             "creating slaves." % ec2_keypair_name )
        return self.ctx.download_ssh_pubkey( ec2_keypair )

    def _populate_ec2_keypair_globs( self, ec2_keypair_globs ):
        super( JenkinsSlave, self )._populate_ec2_keypair_globs( ec2_keypair_globs )
        ec2_keypair_globs.append( JenkinsMaster.ec2_keypair_name( self.ctx ) )

    @fabric_task
    def _setup_build_user( self ):
        """
        Setup a user account that accepts SSH connections from Jenkins such that it can act as a
        Jenkins slave.
        """
        kwargs = dict(
                user=Jenkins.user,
                dir=build_dir,
                ephemeral=self._ephemeral_mount_point( 0 ),
                pubkey=self.__get_master_pubkey( ).strip( ) )

        # Create the build user
        #
        sudo( 'useradd -m -s /bin/bash {0}'.format( Jenkins.user ) )
        self._propagate_authorized_keys( Jenkins.user )

        # Ensure that jenkins@jenkins-master can log into this box as the build user
        #
        sudo( "echo '{pubkey}' >> ~/.ssh/authorized_keys".format( **kwargs ),
              user=Jenkins.user,
              sudo_args='-i' )

        self.setup_repo_host_keys( user=Jenkins.user )

        # Setup working directory for all builds in either the build user's home or as a symlink to
        # the ephemeral volume if available. Remember, the ephemeral volume comes back empty every
        # time the box starts.
        #
        if sudo( 'test -d {ephemeral}'.format( **kwargs ), quiet=True ).failed:
            sudo( 'mkdir {ephemeral}'.format( **kwargs ) )
        chown_cmd = "mount {ephemeral} || true ; chown -R {user}:{user} {ephemeral}".format(
                **kwargs )
        # chown ephemeral storage now ...
        sudo( chown_cmd )
        # ... and every time instance boots. Note that command must work when set -e is in effect.
        self._register_init_command( chown_cmd )
        # link build directory as symlink to ephemeral volume
        sudo( 'ln -snf {ephemeral} {dir}'.format( **kwargs ),
              user=Jenkins.user,
              sudo_args='-i' )

    def __jenkins_labels( self ):
        labels = self.role( ).split( '-' )
        return [ l for l in labels if l not in [ 'jenkins', 'slave' ] ]

    def slave_config_template( self, image, instance_type=None ):
        """
        Returns the slave template, i.e. a fragment of Jenkins configuration that,
        if added to the master's main config file, controls how EC2 instances of this slave box
        are created and managed by the master.

        :param image: the image to boot slave instances from
        :type image: boto.ec2.image.Image

        :return: an XML element containing the slave template
        :rtype: xml.etree.ElementTree.Element
        """
        if instance_type is None:
            instance_type = self.recommended_instance_type( )
        self._set_instance_options( image.tags )
        creation_kwargs = dict( instance_type=instance_type )
        self._populate_instance_spec( image, creation_kwargs )
        return E( 'hudson.plugins.ec2.SlaveTemplate',
                  E.ami( image.id ),
                  # By convention we use the description element as the primary identifier. We
                  # don't need to use the absolute role name since we are not going to mix slaves
                  # from different namespaces:
                  E.description( self.role( ) ),
                  E.zone( self.ctx.availability_zone ),
                  E.securityGroups( self.ctx.to_aws_name( self._security_group_name( ) ) ),
                  E.remoteFS( build_dir ),
                  E.sshPort( '22' ),
                  E.type( snake_to_camel( instance_type, separator='.' ) ),
                  E.labels( ' '.join( self.__jenkins_labels( ) ) ),
                  E.mode( 'EXCLUSIVE' ),
                  E.initScript( 'while ! touch %s/.writable; do sleep 1; done' % build_dir ),
                  E.userData( creation_kwargs.get( 'user_data', '' ) ),
                  E.numExecutors( '1' ),
                  E.remoteAdmin( Jenkins.user ),
                  # Using E.foo('') instead of just E.foo() yields <foo></foo> instead of <foo/>,
                  # consistent with how Jenkins serializes its config:
                  E.rootCommandPrefix( '' ),
                  E.jvmopts( '' ),
                  E.subnetId( '' ),
                  E.idleTerminationMinutes( '30' ),
                  E.iamInstanceProfile( self.get_instance_profile_arn( ) ),
                  E.instanceCap( '1' ),
                  E.stopOnTerminate( 'false' ),
                  E.tags( *[
                      E( 'hudson.plugins.ec2.EC2Tag',
                         E.name( k ),
                         E.value( v ) )
                      for k, v in self._get_instance_options( ).iteritems( )
                      if v is not None ] ),
                  E.usePrivateDnsName( 'false' ) )



def roles( ):
    from cgcloud.jenkins.jenkins_master import JenkinsMaster
    from cgcloud.jenkins.generic_jenkins_slaves import (UbuntuLucidGenericJenkinsSlave,
                                                        Centos5GenericJenkinsSlave,
                                                        Centos6GenericJenkinsSlave,
                                                        Fedora19GenericJenkinsSlave,
                                                        Fedora20GenericJenkinsSlave,
                                                        UbuntuPreciseGenericJenkinsSlave,
                                                        UbuntuTrustyGenericJenkinsSlave)
    from cgcloud.jenkins.cgcloud_jenkins_slave import CgcloudJenkinsSlave
    from cgcloud.jenkins.rpmbuild_jenkins_slaves import (Centos5RpmbuildJenkinsSlave,
                                                         Centos6RpmbuildJenkinsSlave)
    from cgcloud.jenkins.s3am_jenkins_slave import S3amJenkinsSlave
    from cgcloud.jenkins.toil_jenkins_slave import ToilJenkinsSlave
    from cgcloud.jenkins.docker_jenkins_slave import DockerJenkinsSlave
    return sorted( locals( ).values( ), key=lambda cls: cls.__name__ )


def command_classes( ):
    from cgcloud.jenkins.commands import RegisterSlaves
    return sorted( locals( ).values( ), key=lambda cls: cls.__name__ )



from cgcloud.core.generic_boxes import *
from cgcloud.fabric.operations import sudo

from cgcloud.jenkins.jenkins_slave import JenkinsSlave
from cgcloud.core.ubuntu_box import UbuntuBox


class GenericJenkinsSlave( JenkinsSlave ):
    """
    A generic Jenkins slave
    """
    pass


class CentosGenericJenkinsSlave( CentosBox, GenericJenkinsSlave ):
    """
    A generic Jenkins slave for CentOS
    """

    def _list_packages_to_install( self ):
        # TODO: List JRE explicitly (it is already installed on RightScale CentOS images)
        return super( CentosGenericJenkinsSlave, self )._list_packages_to_install( ) + [ ]

    @fabric_task
    def _setup_build_user( self ):
        super( CentosGenericJenkinsSlave, self )._setup_build_user( )
        sudo( "echo 'Defaults:jenkins !requiretty' >> /etc/sudoers" )
        sudo( "echo 'jenkins ALL=(ALL) NOPASSWD: /bin/rpm' >> /etc/sudoers" )
        sudo( "echo 'jenkins ALL=(ALL) NOPASSWD: /usr/bin/yum' >> /etc/sudoers" )

    @fabric_task
    def _post_install_packages( self ):
        super( CentosGenericJenkinsSlave, self )._post_install_packages( )
        # FIXME: These are public but we should rebuild them and host them within our control
        self._yum_local( is_update=False, rpm_urls=[
            'http://public-artifacts.cghub.ucsc.edu.s3.amazonaws.com/custom-centos-packages/python27-2.7.2-cghub.x86_64.rpm',
            'http://public-artifacts.cghub.ucsc.edu.s3.amazonaws.com/custom-centos-packages/python27-devel-2.7.2-cghub.x86_64.rpm',
            'http://public-artifacts.cghub.ucsc.edu.s3.amazonaws.com/custom-centos-packages/python27-setuptools-0.6c11-cghub.noarch.rpm'
        ] )


class Centos5GenericJenkinsSlave( CentosGenericJenkinsSlave, GenericCentos5Box ):
    """
    A generic Jenkins slave for CentOS 5
    """
    pass


class Centos6GenericJenkinsSlave( CentosGenericJenkinsSlave, GenericCentos6Box ):
    """
    A generic Jenkins slave for CentOS 6
    """
    pass


class UbuntuGenericJenkinsSlave( UbuntuBox, GenericJenkinsSlave ):
    """
    A generic Jenkins slave for Ubuntu
    """

    def _list_packages_to_install( self ):
        return super( UbuntuGenericJenkinsSlave, self )._list_packages_to_install( ) + [
            'openjdk-7-jre-headless',
            'gdebi-core' ]  # comes in handy when installing .deb's with dependencies

    @fabric_task
    def _setup_build_user( self ):
        super( UbuntuGenericJenkinsSlave, self )._setup_build_user( )
        sudo( "echo 'Defaults:jenkins !requiretty' >> /etc/sudoers" )
        for prog in ('apt-get', 'dpkg', 'gdebi'):
            sudo( "echo 'jenkins ALL=(ALL) NOPASSWD: /usr/bin/%s' >> /etc/sudoers" % prog )

    def _get_debconf_selections( self ):
        # On Lucid, somehow postfix gets pulled in as a dependency kicking the frontend into
        # interactive mode. The same happens when installing GridEngine.
        return super( UbuntuGenericJenkinsSlave, self )._get_debconf_selections( ) + [
            "postfix postfix/main_mailer_type string 'No configuration'",
            "postfix postfix/mailname string %s" % self.host_name
        ]


class UbuntuLucidGenericJenkinsSlave( UbuntuGenericJenkinsSlave, GenericUbuntuLucidBox ):
    """
    A generic Jenkins slave for Ubuntu 10.04 LTS (EOL April 2015)
    """

    def _setup_package_repos( self ):
        super( UbuntuLucidGenericJenkinsSlave, self )._setup_package_repos( )
        self.__add_git_ppa( )
        self.__add_python_ppa( )

    @fabric_task
    def __add_git_ppa( self ):
        sudo( 'add-apt-repository -y ppa:git-core/ppa' )

    @fabric_task
    def __add_python_ppa( self ):
        sudo( 'apt-add-repository -y ppa:fkrull/deadsnakes/ubuntu' )

    def _list_packages_to_install( self ):
        return super( UbuntuLucidGenericJenkinsSlave, self )._list_packages_to_install( ) + [
            'python2.7',
            'python2.7-dev'
        ]

    def _get_package_substitutions( self ):
        return super( UbuntuLucidGenericJenkinsSlave, self )._get_package_substitutions( ) + [
            ('openjdk-7-jre-headless', 'openjdk-6-jre') ]


class UbuntuPreciseGenericJenkinsSlave( UbuntuGenericJenkinsSlave, GenericUbuntuPreciseBox ):
    """
    A generic Jenkins slave for Ubuntu 12.04 LTS (EOL April 2017)
    """
    pass


class UbuntuTrustyGenericJenkinsSlave( UbuntuGenericJenkinsSlave, GenericUbuntuTrustyBox ):
    """
    A generic Jenkins slave for Ubuntu 14.04 LTS (EOL April 2019)
    """
    pass


class FedoraGenericJenkinsSlave( FedoraBox, GenericJenkinsSlave ):
    """
    A generic Jenkins slave for Fedora
    """

    def _list_packages_to_install( self ):
        return super( FedoraGenericJenkinsSlave, self )._list_packages_to_install( ) + [
            'java-1.7.0-openjdk' ]

    @fabric_task
    def _setup_build_user( self ):
        super( FedoraGenericJenkinsSlave, self )._setup_build_user( )
        sudo( "echo 'Defaults:jenkins !requiretty' >> /etc/sudoers" )
        sudo( "echo 'jenkins ALL=(ALL) NOPASSWD: /bin/rpm' >> /etc/sudoers" )
        sudo( "echo 'jenkins ALL=(ALL) NOPASSWD: /usr/bin/yum' >> /etc/sudoers" )


class Fedora19GenericJenkinsSlave( FedoraGenericJenkinsSlave, GenericFedora19Box ):
    """
    A generic Jenkins slave for Fedora 19
    """
    pass


class Fedora20GenericJenkinsSlave( FedoraGenericJenkinsSlave, GenericFedora20Box ):
    """
    A generic Jenkins slave for Fedora 20
    """
    pass



from cgcloud.core.ubuntu_box import Python27UpdateUbuntuBox
from cgcloud.jenkins.generic_jenkins_slaves import UbuntuTrustyGenericJenkinsSlave
from cgcloud.core.docker_box import DockerBox


class DockerJenkinsSlave( UbuntuTrustyGenericJenkinsSlave, DockerBox, Python27UpdateUbuntuBox ):
    """
    A box for running the cgl-docker-lib builds on. Probably a bit of a misnomer but so far the
    only cgl-docker-lib particular is the dependency on make.
    """

    def _list_packages_to_install( self ):
        return super( DockerJenkinsSlave, self )._list_packages_to_install( ) + [ 'make' ]

    def _docker_users( self ):
        return super( DockerJenkinsSlave, self )._docker_users( ) + [ 'jenkins' ]

    @classmethod
    def recommended_instance_type( cls ):
        return 'm3.large'



from StringIO import StringIO
import time
import re

from fabric.operations import run, put

from cgcloud.core.mesos_box import MesosBox
from cgcloud.core.ubuntu_box import Python27UpdateUbuntuBox
from cgcloud.jenkins.generic_jenkins_slaves import UbuntuTrustyGenericJenkinsSlave
from cgcloud.jenkins.jenkins_master import Jenkins
from cgcloud.core.box import fabric_task
from cgcloud.core.common_iam_policies import s3_full_policy, sdb_full_policy
from cgcloud.core.docker_box import DockerBox
from cgcloud.fabric.operations import sudo, remote_sudo_popen
from cgcloud.lib.util import abreviated_snake_case_class_name, heredoc


class ToilJenkinsSlave( UbuntuTrustyGenericJenkinsSlave,
                        Python27UpdateUbuntuBox,
                        DockerBox,
                        MesosBox ):
    """
    A Jenkins slave suitable for running Toil unit tests, specifically the Mesos batch system and
    the AWS job store. Legacy batch systems (parasol, gridengine, ...) are not yet supported.
    """

    @classmethod
    def recommended_instance_type( cls ):
        return "m3.large"

    def _list_packages_to_install( self ):
        return super( ToilJenkinsSlave, self )._list_packages_to_install( ) + [
            'python-dev', 'gcc', 'make',
            'libffi-dev',  # pynacl -> toil, Azure client-side encryption
            'libcurl4-openssl-dev'  # pycurl -> SPARQLWrapper -> rdflib>=4.2.0 -> cwltool -> toil
        ] + [ 'gridengine-' + p for p in ('common', 'master', 'client', 'exec') ]

    def _get_debconf_selections( self ):
        return super( ToilJenkinsSlave, self )._get_debconf_selections( ) + [
            'gridengine-master shared/gridenginemaster string localhost',
            'gridengine-master shared/gridenginecell string default',
            'gridengine-master shared/gridengineconfig boolean true' ]

    def _post_install_packages( self ):
        super( ToilJenkinsSlave, self )._post_install_packages( )
        self.setup_repo_host_keys( )
        self.__disable_mesos_daemons( )
        self.__install_parasol( )
        self.__patch_distutils( )
        self.__configure_gridengine( )

    @fabric_task
    def __disable_mesos_daemons( self ):
        for daemon in ('master', 'slave'):
            sudo( 'echo manual > /etc/init/mesos-%s.override' % daemon )

    @fabric_task
    def __install_parasol( self ):
        run( "git clone git@github.com:BD2KGenomics/parasol-binaries.git" )
        sudo( "cp parasol-binaries/* /usr/local/bin" )
        run( "rm -rf parasol-binaries" )

    def _get_iam_ec2_role( self ):
        role_name, policies = super( ToilJenkinsSlave, self )._get_iam_ec2_role( )
        role_name += '--' + abreviated_snake_case_class_name( ToilJenkinsSlave )
        policies.update( dict( s3_full=s3_full_policy, sdb_full=sdb_full_policy ) )
        return role_name, policies

    @fabric_task
    def __patch_distutils( self ):
        """
        https://hg.python.org/cpython/rev/cf70f030a744/
        https://bitbucket.org/pypa/setuptools/issues/248/exit-code-is-zero-when-upload-fails
        Fixed in 2.7.8: https://hg.python.org/cpython/raw-file/v2.7.8/Misc/NEWS
        """
        if self._remote_python_version( ) < (2, 7, 8):
            with remote_sudo_popen( 'patch -d /usr/lib/python2.7 -p2' ) as patch:
                patch.write( heredoc( """
                    --- a/Lib/distutils/command/upload.py
                    +++ b/Lib/distutils/command/upload.py
                    @@ -10,7 +10,7 @@ import urlparse
                     import cStringIO as StringIO
                     from hashlib import md5

                    -from distutils.errors import DistutilsOptionError
                    +from distutils.errors import DistutilsError, DistutilsOptionError
                     from distutils.core import PyPIRCCommand
                     from distutils.spawn import spawn
                     from distutils import log
                    @@ -181,7 +181,7 @@ class upload(PyPIRCCommand):
                                     self.announce(msg, log.INFO)
                             except socket.error, e:
                                 self.announce(str(e), log.ERROR)
                    -            return
                    +            raise
                             except HTTPError, e:
                                 status = e.code
                                 reason = e.msg
                    @@ -190,5 +190,6 @@ class upload(PyPIRCCommand):
                                 self.announce('Server response (%s): %s' % (status, reason),
                                               log.INFO)
                             else:
                    -            self.announce('Upload failed (%s): %s' % (status, reason),
                    -                          log.ERROR)
                    +            msg = 'Upload failed (%s): %s' % (status, reason)
                    +            self.announce(msg, log.ERROR)
                    +            raise DistutilsError(msg)""" ) )

    @fabric_task
    def __configure_gridengine( self ):
        """
        Configure the GridEngine daemons (master and exec) and creata a default queue. Ensure
        that the queue is updated to reflect the number of cores actually available.
        """

        ws = re.compile( r'\s+' )
        nl = re.compile( r'[\r\n]+' )

        def qconf( opt, **kwargs ):
            return qconf_dict( opt, kwargs )

        def qconf_dict( opt, d=None, file_name='qconf.tmp' ):
            if d:
                # qconf can't read from stdin for some reason, neither -, /dev/stdin or /dev/fd/0 works
                s = '\n'.join( ' '.join( i ) for i in d.iteritems( ) ) + '\n'
                put( remote_path=file_name, local_path=StringIO( s ) )
                sudo( ' '.join( [ 'qconf', opt, file_name ] ) )
                run( ' '.join( [ 'rm', file_name ] ) )
            else:
                return dict( tuple( ws.split( l, 1 ) )
                                 for l in nl.split( run( 'SGE_SINGLE_LINE=1 qconf ' + opt ) )
                                 if l and not l.startswith( '#' ) )

        # Add the user defined in fname to the Sun Grid Engine cluster.
        qconf( '-Auser', name=Jenkins.user, oticket='0', fshare='0', delete_time='0',
               default_project='NONE' )

        # Adds users to Sun Grid Engine user access lists (ACLs).
        sudo( 'qconf -au %s arusers' % Jenkins.user )

        # Add hosts hostname to the list of hosts allowed to submit Sun Grid Engine jobs and
        # control their behavior only.
        sudo( 'qconf -as localhost' )

        # Remove all currently defined execution hosts
        run( 'for i in `qconf -sel`; do sudo qconf -de $i ; done' )

        # Add an execution host
        qconf( '-Ae', hostname='localhost', load_scaling='NONE', complex_values='NONE',
               user_lists='arusers', xuser_lists='NONE', projects='NONE', xprojects='NONE',
               usage_scaling='NONE', report_variables='NONE' )

        # Add a parallel environment
        qconf( '-Ap', pe_name='smp', slots='999', user_lists='NONE', xuser_lists='NONE',
               start_proc_args='/bin/true', stop_proc_args='/bin/true', allocation_rule='$pe_slots',
               control_slaves='FALSE', job_is_first_task='TRUE', urgency_slots='min',
               accounting_summary='FALSE' )

        # Add a queue, the slots and processors will be adjusted dynamically, by an init script
        qconf( '-Aq', qname='all.q', processors='1', slots='1', hostlist='localhost', seq_no='0',
               load_thresholds='np_load_avg=1.75', suspend_thresholds='NONE', nsuspend='1',
               suspend_interval='00:05:00', priority='0', min_cpu_interval='00:05:00',
               qtype='BATCH INTERACTIVE', ckpt_list='NONE', pe_list='make smp', rerun='FALSE',
               tmpdir='/tmp', shell='/bin/bash', prolog='NONE', epilog='NONE',
               shell_start_mode='posix_compliant', starter_method='NONE', suspend_method='NONE',
               resume_method='NONE', terminate_method='NONE', notify='00:00:60', owner_list='NONE',
               user_lists='arusers', xuser_lists='NONE', subordinate_list='NONE',
               complex_values='NONE', projects='NONE', xprojects='NONE', calendar='NONE',
               initial_state='default', s_rt='INFINITY', h_rt='INFINITY', s_cpu='INFINITY',
               h_cpu='INFINITY', s_fsize='INFINITY', h_fsize='INFINITY', s_data='INFINITY',
               h_data='INFINITY', s_stack='INFINITY', h_stack='INFINITY', s_core='INFINITY',
               h_core='INFINITY', s_rss='INFINITY', h_rss='INFINITY', s_vmem='INFINITY',
               h_vmem='INFINITY' )

        # Enable on-demand scheduling. This will eliminate the long time that jobs spend waiting
        # in the qw state. There is no -Asconf so we have to fake it using -ssconf and -Msconf.
        sconf = qconf( '-ssconf' )
        sconf.update( dict( flush_submit_sec='1', flush_finish_sec='1',
                            schedule_interval='0:0:1' ) )
        qconf_dict( '-Msconf', sconf )

        # Enable immediate flushing of the accounting file. The SGE batch system in Toil uses the
        #  qacct program to determine the exit code of a finished job. The qacct program reads
        # the accounting file. By default, this file is written to every 15 seconds which means
        # that it may take up to 15 seconds before a finished job is seen by Toil. An
        # accounting_flush_time value of 00:00:00 causes the accounting file to be flushed
        # immediately, allowing qacct to report the status of finished jobs immediately. Again,
        # there is no -Aconf, so we fake it with -sconf and -Mconf. Also, the file name has to be
        # 'global'.
        conf = qconf( '-sconf' )
        params = dict( tuple( e.split( '=' ) ) for e in conf[ 'reporting_params' ].split( ' ' ) )
        params[ 'accounting_flush_time' ] = '00:00:00'
        conf[ 'reporting_params' ] = ' '.join( '='.join( e ) for e in params.iteritems( ) )
        qconf_dict( '-Mconf', conf, file_name='global' )

        # Register an init-script that ensures GridEngine uses localhost instead of hostname
        path = '/var/lib/gridengine/default/common/'
        self._register_init_script( 'gridengine-pre', heredoc( """
            description "GridEngine pre-start configuration"
            console log
            start on filesystem
            pre-start script
                echo localhost > {path}/act_qmaster ; chown sgeadmin:sgeadmin {path}/act_qmaster
                echo localhost `hostname -f` > {path}/host_aliases
            end script""" ) )

        # Register an init-script that adjust the queue config to reflect the number of cores
        self._register_init_script( 'gridengine-post', heredoc( """
            description "GridEngine post-start configuration"
            console log
            # I would rather depend on the gridengine daemons but don't know how as they are
            # started by SysV init scripts. Supposedly the 'rc' job is run last.
            start on started rc
            pre-start script
                cores=$(grep -c '^processor' /proc/cpuinfo)
                qconf -mattr queue processors $cores `qselect`
                qconf -mattr queue slots $cores `qselect`
            end script""" ) )

        # Run pre-start script
        for daemon in ('exec', 'master'):
            sudo( '/etc/init.d/gridengine-%s stop' % daemon )
        sudo( "killall -9 -r 'sge_.*'", warn_only=True )  # the exec daemon likes to hang
        self._run_init_script( 'gridengine-pre' )
        for daemon in ('master', 'exec'):
            sudo( '/etc/init.d/gridengine-%s start' % daemon )

        # Run post-start script
        self._run_init_script( 'gridengine-post' )
        while 'execd is in unknown state' in run( 'qstat -f -q all.q -explain a', warn_only=True ):
            time.sleep( 1 )

    def _docker_users( self ):
        return super( ToilJenkinsSlave, self )._docker_users( ) + [ self.default_account( ) ]



from __future__ import absolute_import

from setuptools import setup, find_packages

from version import cgcloud_version, fabric_version

setup(
    name='cgcloud-jenkins',
    version=cgcloud_version,

    author="Hannes Schmidt",
    author_email="hannes@ucsc.edu",
    url='https://github.com/BD2KGenomics/cgcloud',
    description='Setup and manage a Jenkins continuous integration cluster in EC2',

    package_dir={ '': 'src' },
    packages=find_packages( 'src' ),
    namespace_packages=[ 'cgcloud' ],
    install_requires=[
        'cgcloud-lib==' + cgcloud_version,
        'cgcloud-core=='  + cgcloud_version,
        'Fabric==' + fabric_version ] )
